# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'ts_title.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from __future__ import absolute_import
from builtins import object
from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from . import icons_rc

class Ui_ts_title(object):
    def setupUi(self, ts_title):
        if not ts_title.objectName():
            ts_title.setObjectName(u"ts_title")
        ts_title.resize(543, 51)
        self.horizontalLayout = QHBoxLayout(ts_title)
        self.horizontalLayout.setSpacing(12)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(5, -1, -1, -1)
        self.icon = QLabel(ts_title)
        self.icon.setObjectName(u"icon")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.icon.sizePolicy().hasHeightForWidth())
        self.icon.setSizePolicy(sizePolicy)
        self.icon.setMinimumSize(QSize(32, 32))
        self.icon.setPixmap(QPixmap(u":/icons/icons/TSIcon.png"))

        self.horizontalLayout.addWidget(self.icon)

        self.label = QLabel(ts_title)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setPointSize(20)
        self.label.setFont(font)

        self.horizontalLayout.addWidget(self.label)


        self.retranslateUi(ts_title)

        QMetaObject.connectSlotsByName(ts_title)
    # setupUi

    def retranslateUi(self, ts_title):
        ts_title.setWindowTitle(QCoreApplication.translate("ts_title", u"Form", None))
        self.icon.setText("")
        self.label.setText(QCoreApplication.translate("ts_title", u"Title", None))
    # retranslateUi




import glob
import os
import subprocess

import publisher.settings as settings
from publisher.translations.utils import get_locale_directory


def build_mo_files():

    for locale in settings.LOCALE_DIRECTORIES.keys():

        locale_directory = get_locale_directory(locale)
        if os.path.exists(locale_directory):
            django_po_file = os.path.join(locale_directory, 'django.po')
            django_mo_file = os.path.join(locale_directory, 'django.mo')

            if os.path.exists(django_po_file):
                os.remove(django_po_file)

            po_files = glob.glob(os.path.join(locale_directory, '*.po'))

            subprocess.check_output(['msgcat'] + po_files + ['-o', django_po_file])
            subprocess.check_output(['msgfmt', '-o', django_mo_file, django_po_file])



from __future__ import absolute_import, division, print_function, unicode_literals

import getpass
import os
import shutil
import subprocess
import sys

from Crypto.PublicKey import RSA
import publisher.settings as settings
from publisher.exceptions import ValidationError
from publisher.touchsurgery import TouchSurgery
from publisher.utils import WorkingDirectory, get_command_output, get_input, get_platform, \
    call_command_and_print_exception


def setup_users_machine():
    """
    User Setup for publish
        1. Check if git is installed
        2. Check if Node.js is installed
        3. Install studio-report app
        4. Setup SSH
        5. Cloning the repo and git pull
    """
    if not git_installed():
        win_msg = 'Please install Github for windows from https://desktop.github.com/ before coming back and ' \
                  'continuing and running setup again.'
        mac_msg = 'Please install Git for Mac from https://git-scm.com/download/mac before running setup again.'

        print("%s" % mac_msg if 'darwin' in get_platform() else win_msg)
        sys.exit(1)

    if not node_installed():
        print("""
            Please install Node.js for your platform from: https://nodejs.org/en/download/ before running setup again.
        """)
        sys.exit(1)

    if not os.path.isdir("{report_folder}/node_modules".format(report_folder=settings.REPORT_FOLDER)):
        print("Installing studio-report tool.")
        install_studio_report_tool()

    print('Generating key and configuring user profile')
    check_and_create_directory(settings.SSH_DIRECTORY_PATH)
    sys.stdout.flush()

    login_success = False
    get_username()
    while not login_success:
        if login(get_email(), get_password(), create_rsa_return_pub_key()):
            login_success = True

    print("Configuring ssh config file ")
    check_and_create_file(settings.SSH_CONFIG_PATH)
    if not has_private_key():
        raise RuntimeError("Unable to proceed without a key. Please contact Hansel before trying again")
    configure_ssh_config()

    print("Installing and configuring git lfs")
    check_brew_installed()
    install_git_lfs()
    configure_git_lfs()

    check_and_create_directory(settings.GIT_DIRECTORY)
    for repository, checkout_directory in get_repositories_to_checkout():
        clone_repo(repository, checkout_directory)
        pull_repo(checkout_directory)


def get_repositories_to_checkout():
    return [(settings.PROCEDURE_REPOSITORY, settings.PROCEDURE_CHECKOUT_DIRECTORY),
            (settings.CHANNELS_REPOSITORY, settings.CHANNELS_CHECKOUT_DIRECTORY),
            (settings.TRANSLATIONS_REPOSITORY, settings.TRANSLATIONS_CHECKOUT_DIRECTORY)]


def git_installed():
    try:
        subprocess.check_output(['git', '--version'])
    except OSError:
        print("Git not installed")
        return False
    return True


def node_installed():
    try:
        subprocess.check_output(['node', '--version'])
    except OSError:
        print("Node.js is not installed")
        return False
    return True


def install_studio_report_tool():
    npm_command = ["npm.cmd", "i"] if sys.platform == "win32" else ["npm", "install"]

    p = subprocess.Popen(
        npm_command, cwd=settings.REPORT_FOLDER, stdout=subprocess.PIPE
    )
    (output) = p.communicate()
    print(output)
    p.wait()


def get_username(all_repos=True):
    print("Please enter your name here:")
    sys.stdout.flush()
    username = get_input()
    if all_repos:
        subprocess.check_output(['git', 'config', '--global', 'user.name', username])
    else:
        subprocess.check_output(['git', 'config', 'user.name', username])


def get_email(all_repos=True):
    print("Please enter your touch surgery email here:")
    sys.stdout.flush()
    email = get_input()
    if all_repos:
        subprocess.check_output(['git', 'config', '--global', 'user.email', email])
    else:
        subprocess.check_output(['git', 'config', 'user.email', email])
    return email


def get_password():
    print("Please enter your touch surgery password here:")
    sys.stdout.flush()
    password = getpass.getpass()
    return password


def has_private_key():
    """Check whether the user has the correct private key
    """
    return os.path.exists(settings.RSA_PUBLIC_KEY_PATH)


def configure_ssh_config():
    """Creates and sets up an ssh config file, or appends the necessary entry to an existing one
    """
    shutil.copyfile(os.path.expanduser(settings.SSH_CONFIG_PATH), os.path.expanduser(settings.SSH_CONFIG_PATH + '.bak'))

    obsolete_stanza = (
        'Host {0}\n '
        'User ubuntu\n '
        'IdentitiesOnly true\n '
        'IdentityFile ~/.ssh/touchsurgery-studio.pem\n'
    ).format(settings.STUDIO_GIT_PATH)

    ssh_config_stanza = (
        'Host {0}\n'
        ' StrictHostKeyChecking no\n'
        ' User git\n'
        ' IdentitiesOnly true\n'
        ' IdentityFile {1}\n'
    ).format(settings.STUDIO_GIT_PATH, settings.RSA_PRIVATE_KEY_PATH)

    try:

        with open(os.path.expanduser(settings.SSH_CONFIG_PATH), "r") as config_file:
            current_config_text = config_file.read()
        ssh_config_missing = ssh_config_stanza not in current_config_text
        obsolete_stanza_present = obsolete_stanza in current_config_text

        # Remove outdated config info
        if obsolete_stanza_present:
            current_config_text = current_config_text.replace(obsolete_stanza, '')
            with open(os.path.expanduser(settings.SSH_CONFIG_PATH), "w") as config_file:
                config_file.write(current_config_text)

        # Add relevant config info
        if ssh_config_missing:
            with open(os.path.expanduser(settings.SSH_CONFIG_PATH), "a+") as config_file:
                config_file.write('\n' + '\n' + ssh_config_stanza)

    except Exception:
        print("Unable to configure the ssh config")
        raise


def check_brew_installed():
    """ Get Macs ready to brew
    """
    if 'darwin' in get_platform():
        output, _ = get_command_output(['brew', 'help'])
        if 'usage' not in output.lower():
            raise Exception("Please install Brew from here: https://brew.sh/")


def install_git_lfs():
    """Install git lfs
    """
    if 'darwin' in get_platform():
        output, _ = get_command_output(['which', 'git-lfs'])
        if 'usr' not in output.lower():
            call_command_and_print_exception(['brew', 'install', 'git-lfs'], "brew lfs install failure")

    call_command_and_print_exception(['git', 'lfs', 'install'], "lfs install failure")


def clone_repo(repository, directory):
    if not os.path.exists(directory):
        call_command_and_print_exception(['git', 'lfs', 'clone', repository, directory], "Clone repo failure")
    else:
        print("Not cloning repository: {0} already exists".format(directory))


def pull_repo(directory):
    with WorkingDirectory(directory):
        call_command_and_print_exception(['git', 'lfs', 'pull', 'origin', 'master'], "Git pull failure")


def check_and_create_directory(path):
    try:
        if not os.path.exists(path):
            os.mkdir(path)
    except Exception:
        print("Could not find or create the directory")
        raise ValidationError


def check_and_create_file(path):
    try:
        if not os.path.exists(path):
            subprocess.check_output(['touch', path])
    except Exception as e:
        print("Could not find or create the file")
        print(e)
        raise ValidationError


def configure_git_lfs():
    """Set relevant  lfs settings
    """
    call_command_and_print_exception(['git', 'config', '--global', 'lfs.url',
                                      'https://live.touchsurgery.com/api/v3/lfs'], "lfs config failure")
    call_command_and_print_exception(['git', 'config', '--global', 'lfs.activitytimeout', '60'], "lfs config failure")


def create_rsa_return_pub_key():
    private_key = settings.RSA_PRIVATE_KEY_PATH
    public_key = settings.RSA_PUBLIC_KEY_PATH

    key = RSA.generate(2048)
    if os.path.exists(private_key):
        os.chmod(private_key, 0o0600)

    with open(private_key, 'wb') as rsa_pri:
        os.chmod(private_key, 0o0600)
        rsa_pri.write(key.exportKey('PEM'))

    pubkey = key.publickey()
    with open(public_key, 'wb') as rsa_pub:
        pub_key = pubkey.exportKey('OpenSSH')
        rsa_pub.write(pubkey.exportKey('OpenSSH'))

    return pub_key.split()[1]


def login(email, password, pub_key):
    """Verify TouchSurgery user here with rsa key and TouchSurgery login
    """
    login_instance = TouchSurgery()
    if not login_instance.login(email, password):
        return False
    if not login_instance.upload_key(pub_key):
        print('Your rsa key is invalid, please try running setup again or contact pipeline.')
        raise ValidationError
    return True



from __future__ import division

from builtins import str
from past.utils import old_div
import glob
import os
import shutil
from Qt import QtWidgets, QtCore, QtGui

from publisher.gui.delegates import CheckBoxDelegate, ComboBoxDelegate
from publisher.procedure_content import build_channel_list, save_channel_changes
from publisher.processing.data_sources.organisation import get_data_from_channel
from publisher.processing.procedure import update_channel, build_channel
from publisher.settings import CHANNEL_TYPES, CHANNELS_ASSET_DIRECTORY, ASSET_NAME_PREFIX

from .models import ChannelTableModel
from .ui.channel_editor import Ui_channel_editor
from .widgets import SpinnerOverlay


class ChannelEditor(QtWidgets.QWidget, Ui_channel_editor):

    def __init__(self, parent=None):
        super(ChannelEditor, self).__init__(parent)
        self.new_images = {}
        self.channel_model = ChannelTableModel(self)
        self.channel_model.validation_error.connect(self._display_error)
        self.setupUi(self)
        self.channel_tbl.addAction(self.add_row)
        self.channel_tbl.addAction(self.delete_row)
        self.channel_tbl.setModel(self.channel_model)
        self.channel_tbl.setItemDelegateForColumn(self.channel_model.header.index('eula_required'),
                                                  CheckBoxDelegate(self))
        self.channel_tbl.setItemDelegateForColumn(self.channel_model.header.index('type'),
                                                  ComboBoxDelegate(self, CHANNEL_TYPES))
        self.loading_overlay = SpinnerOverlay(self.channel_tbl)
        # Pyside vs PySide2 incompatability not managed correctly by Qt Shim
        if 'setSectionResizeMode' in dir(self.channel_tbl.horizontalHeader()):
            self.channel_tbl.horizontalHeader().setSectionResizeMode(1, QtWidgets.QHeaderView.Stretch)
        else:
            self.channel_tbl.horizontalHeader().setResizeMode(1, QtWidgets.QHeaderView.Stretch)

        self._display_badge_banner_widget()
        self.load_channel_data()
        self._connect_signals()

    @QtCore.Slot()
    def on_submit_btn_clicked(self):
        data = self.channel_model.edited_channels
        errors = self._validate_submit(data)
        if errors:
            self._display_error(errors)
            return
        message = self.msg.toPlainText()
        self.data_saver = SaveChannelDataWorker(data, message, self.new_images)
        self.data_saver.done.connect(self._on_write_completed)
        self.data_saver.error.connect(self._display_error)
        self.loading_overlay.show()
        self.data_saver.start()

    @QtCore.Slot()
    def on_add_row_triggered(self):
        data = {'code': '', 'name': '', 'type': CHANNEL_TYPES[0], 'eula_required': False, 'new': True}
        row = self.channel_tbl.selectedIndexes()[0].row()
        self.channel_model.add_channel(data, row)

    @QtCore.Slot()
    def on_delete_row_triggered(self):
        row = self.channel_tbl.selectedIndexes()[0].row()
        self.channel_model.delete_row(row)

    def load_channel_data(self):
        self._enable_input(False)
        self.loading_overlay.show()
        self.data_loader = LoadChannelDataWorker()
        self.data_loader.channel_data.connect(self.channel_model.add_channel)
        self.data_loader.done.connect(self._on_data_loaded)
        self.data_loader.start()

    def resizeEvent(self, event):
        self._display_channel_images()
        super(ChannelEditor, self).resizeEvent(event)

    def _connect_signals(self):
        self.display_images.clicked.connect(self._display_badge_banner_widget)
        sel_model = self.channel_tbl.selectionModel()  # store in variable, otherwise application crashes
        sel_model.selectionChanged.connect(self._display_channel_images)
        self.badge.image_dropped.connect(self._change_image)
        self.banner.image_dropped.connect(self._change_image)

    def _change_image(self, image):
        row = self.channel_tbl.currentIndex().row()
        if row >= 0:
            name = self.sender().objectName()
            code = self.channel_model.mylist[row]['code']
            new_name = ASSET_NAME_PREFIX + code + name
            new_image = os.path.join(CHANNELS_ASSET_DIRECTORY, new_name + '.*')
            if glob.glob(new_image):
                reply = QtWidgets.QMessageBox.question(self, 'Overwrite?', '\nOverwrite existing {n}?\n'.format(n=name),
                                                       QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
                if reply == QtWidgets.QMessageBox.No:
                    return
            if not self.new_images.get(code):
                self.new_images[code] = {'badge': '', 'banner': ''}
            self.new_images[code][name] = image
            self._show_image_in_graphics_view(image, eval('self.' + name))
            self.channel_model.mylist[row]['changed'] = True

    def _display_badge_banner_widget(self):
        if self.display_images.isChecked():
            self.badge_banner_widget.show()
            self._expand_window(True)
            if self.channel_tbl.selectedIndexes():
                self._display_channel_images()
        else:
            self._expand_window(False)
            self.badge_banner_widget.hide()

    def _expand_window(self, expand=True):
        top_margin = self.layout().getContentsMargins()[1]
        if expand:
            height = self.height() + self.badge_banner_widget.height() * 2 + top_margin
        else:
            height = self.height() - self.badge_banner_widget.height() - old_div(top_margin, 2)
        self.resize(self.width(), height)

    def _display_channel_images(self):
        row = self.channel_tbl.currentIndex().row()
        if row >= 0 and self.badge_banner_widget.isVisible():
            code = self.channel_model.mylist[row]['code']
            assets = self._unpack_asset_data(row)
            badge = self._get_asset_path(assets, 'badge')
            banner = self._get_asset_path(assets, 'banner')
            if code in list(self.new_images.keys()):
                badge = self.new_images[code]['badge'] if self.new_images[code]['badge'] else badge
                banner = self.new_images[code]['banner'] if self.new_images[code]['banner'] else badge
            self._show_image_in_graphics_view(badge, self.badge)
            self._show_image_in_graphics_view(banner, self.banner)

    def _unpack_asset_data(self, row):
        asset_list = self.channel_model.mylist[row]['assets']
        assets = {}
        for item in asset_list:
            for key, val in item.items():
                assets[key] = val
        return assets

    def _get_asset_path(self, asset_data, asset_name):
        if asset_data.get(asset_name):
            path = glob.glob(os.path.join(CHANNELS_ASSET_DIRECTORY, asset_data[asset_name]['name'] + '.*'))
            if path:
                return path[0].replace(r'/', os.sep)
        return None

    def _show_image_in_graphics_view(self, image, view):
        scene = QtWidgets.QGraphicsScene()
        if image:
            item = QtWidgets.QGraphicsPixmapItem(QtGui.QPixmap(image))
            scene.addItem(item)
        view.setScene(scene)
        view.fitInView(scene.sceneRect(), QtCore.Qt.KeepAspectRatio)

    def _on_data_loaded(self):
        self.loading_overlay.hide()
        self._enable_input()

    def _on_write_completed(self):
        self.loading_overlay.hide()
        self._enable_input()

    def _display_error(self, message):
        QtWidgets.QMessageBox.critical(self, "Error", "Error: {}".format(message))
        self.loading_overlay.hide()
        self._enable_input()

    def _enable_input(self, state=True):
        self.msg.setEnabled(state)
        self.channel_tbl.setEnabled(state)
        self.submit_btn.setEnabled(state)
        self.display_images.setEnabled(state)

    def _validate_submit(self, data):
        if self.msg.toPlainText() == '':
            return "Submit comment must not be blank"
        for item in data:
            if not item.get('name'):
                return "Name is blank for channel with code {}".format(item.get('code'))
            if not item.get('code'):
                return "Code is blank for channel with name {}".format(item.get('name'))


class LoadChannelDataWorker(QtCore.QThread):

    channel_data = QtCore.Signal(dict)
    done = QtCore.Signal()

    def run(self):
        for channel in build_channel_list():
            data = get_data_from_channel(channel)
            self.channel_data.emit(data)
        self.done.emit()


class SaveChannelDataWorker(QtCore.QThread):

    done = QtCore.Signal()
    error = QtCore.Signal(str)

    def __init__(self, data, msg, new_images):
        try:
            self.data = data
            self.message = msg
            self.new_images = new_images
            super(SaveChannelDataWorker, self).__init__()
        except Exception as e:
            message = str(e)
            self.error.emit(message)

    def run(self):
        try:
            for channel in self.data:
                if channel['code'] in list(self.new_images.keys()):
                    self._copy_and_rename_assets(channel['code'], self.new_images[channel['code']])
                if channel.get('new', False):
                    channel_data = {'code': channel['code'], 'name': channel['name'], 'type': channel['type'],
                                    'eula': channel['eula_required']}
                    build_channel(channel_data)
                else:
                    updates = {'name': channel['name'], 'type': channel['type'], 'eula': channel['eula_required']}
                    update_channel(channel['code'], updates)
            save_channel_changes(self.message)
            self.done.emit()
        except Exception as e:
            message = str(e)
            self.error.emit(message)

    def _copy_and_rename_assets(self, code, assets):
        for name, asset in assets.items():
            if asset:
                new_name = ASSET_NAME_PREFIX + code + name
                new_asset = os.path.join(CHANNELS_ASSET_DIRECTORY, new_name + os.path.splitext(asset)[-1])
                existing_assets = glob.glob(os.path.join(CHANNELS_ASSET_DIRECTORY, new_name + '.*'))
                for file in existing_assets:
                    os.remove(file)
                shutil.copy(asset, new_asset)



from __future__ import absolute_import, division, print_function, unicode_literals
from io import open

from abc import abstractmethod

import os

from ruamel.yaml import YAMLError, safe_load

import publisher.settings as settings
from publisher.exceptions import SingleProceduresNotFoundError, MissingYamlFileError, ValidationError, \
    CombinedValidationError, InvalidYamlFileError
from publisher.utils import get_content_directories
import yamale
from yamale import YamaleError
from ruamel.yaml.scanner import ScannerError
from .settings import PHASE_SCHEMA_PATH, PROCEDURE_SCHEMA_PATH, INFO_CARD_SCHEMA_PATH

class Validator(object):

    def __init__(self, working_directory):
        self.working_directory = working_directory

    @abstractmethod
    def validate(self):
        pass


class SingleProcedureValidator(Validator):

    def validate(self):
        procedure_directories = get_content_directories(self.working_directory)

        if len(procedure_directories) != 1:
            print(procedure_directories)
            raise SingleProceduresNotFoundError()


class YamlFileValidator(Validator):

    def __init__(self, yaml_file, schema):
        self.yaml_file = yaml_file
        self.schema = schema

    def validate(self):

        try:
            data = yamale.make_data(self.yaml_file, parser="ruamel")
        except IOError:
            raise MissingYamlFileError(self.yaml_file)
        except ScannerError:
            raise InvalidYamlFileError(
                "Invalid yaml syntax in {}".format(self.yaml_file)
            )

        try:
            yamale.validate(self.schema, data, strict=True)
        except YamaleError as yaml_error:
            invalid_keys = []
            issues = str(yaml_error).split("\n\t")[1:]
            for issue in issues:
                key = issue.split(":")[0]
                if key not in invalid_keys:
                    invalid_keys.insert(0, key)
            raise InvalidYamlFileError(
                "{}\nThe following keys have bad values or are missing:\n{}".format(
                    self.yaml_file, "\n".join(["\t - {}".format(key) for key in invalid_keys])
                )
            )


class ProcedureYamlFileValidator(YamlFileValidator):

    def __init__(self, procedure_directory):
        yaml_file = os.path.join(procedure_directory, "procedure.yml")

        schema = yamale.make_schema(PROCEDURE_SCHEMA_PATH)
        with open(INFO_CARD_SCHEMA_PATH) as include_yaml:
            info_card_includes = safe_load(include_yaml)
        schema.add_include(info_card_includes)

        super(ProcedureYamlFileValidator, self).__init__(yaml_file, schema)


class PhaseYamlFileValidator(YamlFileValidator):

    def __init__(self, phase_directory):
        yaml_file = os.path.join(phase_directory, 'phase.yml')

        schema = yamale.make_schema(PHASE_SCHEMA_PATH)
        with open(INFO_CARD_SCHEMA_PATH) as include_yaml:
            info_card_includes = safe_load(include_yaml)
        schema.add_include(info_card_includes)

        super(PhaseYamlFileValidator, self).__init__(yaml_file, schema)

class AssetDirectoryValidator(Validator):

    def __init__(self, base_directory):
        self.asset_directory = os.path.join(base_directory, 'assets')

    def validate(self):
        if not os.path.exists(self.asset_directory):
            os.mkdir(self.asset_directory)


def validate_procedure_directory():
    SingleProcedureValidator(settings.PROCEDURE_CHECKOUT_DIRECTORY).validate()


def validate_save():

    validate_procedure_directory()

    procedure_directory = os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY,
                                       get_content_directories(settings.PROCEDURE_CHECKOUT_DIRECTORY)[0])

    validators = [ProcedureYamlFileValidator(procedure_directory), AssetDirectoryValidator(procedure_directory)]

    phase_directories = [os.path.join(procedure_directory, d) for d in get_content_directories(procedure_directory)]

    validators += [PhaseYamlFileValidator(d) for d in phase_directories]
    validators += [AssetDirectoryValidator(d) for d in phase_directories]

    errors = []

    for v in validators:
        try:
            v.validate()
        except ValidationError as e:
            errors.append(e)

    if len(errors) > 0:
        raise CombinedValidationError(errors)



from __future__ import absolute_import, division, print_function, unicode_literals
from publisher.exceptions import UpdateNoteError
import subprocess

import publisher.settings as settings
from publisher.utils import WorkingDirectory, get_command_output


def clear_similar_historical_notes(commit, commit_history):

    similar_commits = commit.filter_similar_commits(commit_history)

    for similar_commit in similar_commits:
        update_note('', similar_commit.id)

    push_notes()


def edit_note(commit):
    update_note(commit.note, commit.id)
    push_notes()


def update_note(note, commit_id):
    """ Use the commit object obtained from the get notes command to overwrite a previous note
    """
    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        try:
            output, error = get_command_output(
                ['git', 'notes', 'add', '-f', '-m', note, commit_id])
        except subprocess.CalledProcessError:
            print("Could not change the distribution group for the specified commit")


def push_notes():
    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        print('Pushing ref/notes to origin')
        output, error = get_command_output(['git', 'push', '-q', 'origin', 'refs/notes/*'])
        print(output, error)
        if "Traceback" in error or "Failed: Please fix" in error:
            raise UpdateNoteError(error)



from __future__ import absolute_import, division, print_function, unicode_literals


class ContentMissingError(Exception):
    pass


class ChannelError(Exception):
    pass


class UpdateNoteError(Exception):
    pass


class ValidationError(Exception):
    message = ""
    def __init__(self):
        super(ValidationError, self).__init__(self.message)


class SingleProceduresNotFoundError(ValidationError):
    message = 'An incorrect number of procedures were found in the procedure folder.  ' \
              'There should be exactly one procedure folder in your repository at a time.'


class PublishProcedureError(ValidationError):
    def __init__(self, *args):
        self.message = self.message.format(*args)
        super(PublishProcedureError, self).__init__()


class LocalesNotSpecifiedError(PublishProcedureError):
    message = 'Locales not specified in procedure yaml.'


class InvalidLocaleError(PublishProcedureError):
    message = 'Locale is not supported: {0}'


class MissingAssetDirectoryError(PublishProcedureError):
    message = 'Asset folder missing from {0}'


class MissingYamlFileError(PublishProcedureError):
    message = 'Missing Yaml file {0}'


class InvalidYamlFileError(PublishProcedureError):
    message = 'Invalid Yaml file {0}'


class ProcedureCreationError(PublishProcedureError):
    message = 'Procedure could not be created'


class CombinedValidationError(ValidationError):
    def __init__(self, errors):
        self.errors = errors
        self.message = '\n'.join(map(str, self.errors))
        super(CombinedValidationError, self).__init__()


class ChannelExistsError(ChannelError):
    pass


class ChannelMissingError(ChannelError):
    pass



from __future__ import absolute_import, division, print_function, unicode_literals


class Commit(object):

    NOTYPE = 0
    PUBLIC = 1
    PRIVATE = 2
    TESTING = 3

    def __init__(self, commit_id, comment, author, note):
        self.id = commit_id
        self.comment = comment
        self.author = author
        self.note = note

    def __str__(self):
        return '\n'.join([self.comment,
                          'Author: {0}'.format(self.author),
                          'Current distribution group: {0}'.format(self.note if self.note else 'Not published'),
                          '----------------------------'])

    @property
    def commit_type(self):

        if self.note is None:
            return self.NOTYPE

        note = self.note.lower()

        if "public" in note:
            return self.PUBLIC

        if "ts-testing" in note:
            return self.TESTING

        return self.PRIVATE

    def copy_with_new_note(self, new_note):
        return Commit(self.id, self.comment, self.author, new_note)

    def filter_similar_commits(self, commits):
        def is_note_similar(c):
            return c.commit_type == self.commit_type and c.note is not None

        return [c for c in commits if is_note_similar(c)]



import os.path
from shutil import copyfile
from io import open

from publisher.processing.data_sources.asset import convert_files
from publisher.processing.data_sources.overview import get_overview_and_devices
from publisher.processing.data_sources.eula import copy_eula_to_procedure
from publisher.processing.data_sources.organisation import get_organisation_from_channel, get_data_from_channel
from publisher.processing.data_sources.author import get_author_from_overview
from publisher.processing.data_sources.ftrack import get_procedure_details, get_procedure_phase_list, get_phase_data
from publisher.processing.data_sources.step import get_step_data
from publisher.processing.data_sources.thumbnail import update_thumbnails, update_phase_thumbnails, \
    create_phase_thumbnail, create_procedure_thumbnails
from publisher.processing.data_sources.utils import YamlObject
from publisher.processing.models import Procedure, Phase, Channel
from publisher.processing.utils.file import ProcedureFileManager, PhaseFileManager, DeliveryFileManager
import publisher.settings as settings


def create_procedure(procedure_code):

    procedure = Procedure()
    procedure.code = procedure_code

    phase = Phase()
    phase.code = "{0}_010".format(procedure_code)
    procedure.phases.append(phase)

    procedure_files = ProcedureFileManager(procedure)

    create_yaml_file(procedure, procedure_files.procedure_file)
    create_yaml_file(phase, procedure_files.phase_files[0].phase_file)

    return procedure, procedure_files


def create_procedure_from_production(procedure_code):
    procedure, procedure_files = initialize_procedure(procedure_code)

    create_yaml_file(procedure, procedure_files.procedure_file)
    for phase in procedure.phases:
        phase_files = next(pf for pf in procedure_files.phase_files if pf.phase_code == phase.code)
        create_yaml_file(phase, phase_files.phase_file)

    setup_procedure_files(procedure, procedure_files)

    return procedure, procedure_files


def create_phase(procedure_code, phase_code):
    procedure, procedure_files = initialize_procedure(procedure_code)
    for phase in procedure.phases:
        if phase.code == phase_code:
            phase_files = next(pf for pf in procedure_files.phase_files if pf.phase_code == phase.code)
            create_yaml_file(phase, phase_files.phase_file)

    setup_phase_files(procedure, phase_code)

    return procedure, procedure_files


def initialize_procedure(procedure_code):
    procedure = build_procedure_object(procedure_code)
    procedure_files = ProcedureFileManager(procedure)
    load_existing_locales(procedure, procedure_files)

    return procedure, procedure_files


def load_existing_locales(procedure, procedure_files):
    yaml = YamlObject()
    if os.path.exists(procedure_files.procedure_file):
        with open(procedure_files.procedure_file, "r", encoding="utf-8") as yaml_stream:
            existing_procedure = yaml.load(yaml_stream)
        procedure.locales = existing_procedure.get("locales", [])


def initialize_phase(phase_data):
    phase = build_phase_object(phase_data)
    phase_files = PhaseFileManager(phase, phase_data['procedure_code'])

    return phase, phase_files


def initialize_channel(channel_data):
    channel = build_channel_object(channel_data)
    channel_file = os.path.join(settings.CHANNELS_CHECKOUT_DIRECTORY, channel.code + ".yml")

    return channel, channel_file


def build_procedure_object(procedure_code):
    procedure = Procedure()
    procedure.code = procedure_code
    procedure.name, procedure.specialties, procedure.channel, procedure.labels, vbs = get_procedure_details(procedure_code)
    procedure.vbs = vbs
    phases_data = get_procedure_phase_list(procedure_code)
    procedure.phases = [build_phase_object(p) for p in phases_data]

    return procedure


def build_phase_object(phase_data):

    phase = Phase()

    phase.procedureCode = phase_data['procedure_code']
    phase.code = phase_data['phase_code']
    phase.name = phase_data['phase_name']
    phase.countryRestriction = ",".join([country for country in phase_data['geo_restrictions']])
    # phase.released_as = phase_data['released_as']
    # phase.vbs = is_vbs(phase_data['vbs'])

    return phase


def build_channel_object(channel_data):
    channel = Channel()
    channel.code = channel_data["code"]
    channel.name = channel_data["name"]
    channel.type = channel_data["type"]
    channel.eula = channel_data["eula"]

    return channel


def build_procedure(procedure_code, build_phases=False, graphics=True,
                    pip_graphics=True, widget_graphics=True, thumbnails=True,
                    step_numbers=True, info_step=True):

    print("-- Updating procedure: %s --" % procedure_code)
    procedure, procedure_files = initialize_procedure(procedure_code)
    procedure.overview, procedure.devices = get_overview_and_devices(procedure_code, procedure_files.asset_directory)

    procedure.organisation = get_organisation_from_channel(procedure.channel)
    procedure.author = get_author_from_overview(procedure.overview)
    procedure.eulaFile = copy_eula_to_procedure(procedure_code, procedure_files.asset_directory)

    if thumbnails:
        update_thumbnails(procedure, procedure_files.asset_directory)

    create_yaml_file(procedure, procedure_files.procedure_file)

    if build_phases:
        for phase in procedure.phases:
            phase_files = next(pf for pf in procedure_files.phase_files if pf.phase_code == phase.code)
            build_phase(phase, phase_files, graphics, pip_graphics,
                        widget_graphics, thumbnails, step_numbers=step_numbers,
                        info_step=info_step)


def build_single_phase(phase_code, graphics=True, pip_graphics=True,
                       widget_graphics=True, thumbnails=True, step_numbers=True,
                       info_step=True, csv_version=None):

    phase_data = get_phase_data(phase_code)

    phase, phase_files = initialize_phase(phase_data)

    build_phase(phase, phase_files, graphics, pip_graphics, widget_graphics,
                thumbnails, step_numbers=step_numbers, info_step=info_step,
                csv_version=csv_version)


def build_phase(phase, phase_files, graphics, pip_graphics, widget_graphics,
                thumbnails, step_numbers=True, info_step=True,
                supported_app="touchsurgery", csv_version=None):

    print("-- Updating phase: %s --" % phase.code)
    phase.learnObjectives, phase.testObjectives = get_step_data(phase.code, csv_version)
    phase.phaseDir = phase_files.asset_directory

    phase.supported_app = supported_app
    phase.stepNumbers = step_numbers
    phase.infoStep = info_step

    create_yaml_file(phase, phase_files.phase_file)

    if thumbnails:
        update_phase_thumbnails(phase, phase_files.asset_directory)

    if graphics or widget_graphics or isinstance(pip_graphics, dict):
        convert_files(
            phase,
            phase_files.base_directory,
            graphics,
            pip_graphics,
            widget_graphics,
            csv_version,
        )


def update_phase_name(phase_code, new_name):
    """Update the procedure's name
    """
    phase_data = get_phase_data(phase_code)

    phase, phase_files = initialize_phase(phase_data)
    phase.name = new_name

    create_yaml_file(phase, phase_files.phase_file)


def build_channel(channel_data):
    channel, channel_file = initialize_channel(channel_data)
    create_yaml_file(channel, channel_file)
    return channel, channel_file


def update_channel(code, updates):
    channel_data = get_data_from_channel(code)
    for key in updates:
        channel_data[key] = updates[key]
    build_channel(channel_data)


def setup_templates(procedure_files):
    source_overview = os.path.join(settings.TEMPLATE_DIRECTORY, "overview.yml")
    source_devices = os.path.join(settings.TEMPLATE_DIRECTORY, "devices.yml")
    dest_overview = os.path.join(procedure_files.overview_directory, "overview.yml")
    dest_devices = os.path.join(procedure_files.devices_directory, "devices.yml")
    copyfile(source_overview, dest_overview)
    copyfile(source_devices, dest_devices)


def setup_procedure_files(procedure, procedure_files):
    setup_templates(procedure_files)
    if os.path.isdir(settings.BASE_DATA_DIR):
        delivery_files = DeliveryFileManager(procedure.code, procedure.phases)
        create_procedure_thumbnails(delivery_files.proc_thumbnail_directory, None, None)
        for phase in delivery_files.phase_directories:
            create_phase_thumbnail(delivery_files.phase_directories[phase]["thumbnail_directory"], None)


def setup_phase_files(procedure, phase_code):
    if os.path.isdir(settings.BASE_DATA_DIR):
        delivery_files = DeliveryFileManager(procedure.code, procedure.phases)
        phase = delivery_files.phase_directories[phase_code]
        create_phase_thumbnail(phase["thumbnail_directory"], None)


def create_yaml_file(obj, output_file):

    yaml = YamlObject()

    with open(output_file, "wb") as yaml_stream:
        obj.dump(yaml, yaml_stream)

    # Check yaml validity by loading file
    with open(output_file, "r", encoding="utf-8") as yaml_stream:
        yaml.load(yaml_stream)


def is_vbs(vbs_input):
    if vbs_input.lower() == "y":
        return True

    return False



from __future__ import absolute_import, division, print_function, unicode_literals

from publisher.procedure_content import build_procedure_list, build_phase_list, has_unstaged_changes, reset_repo, \
    change_procedure, save_working_changes, get_commits_for_procedure, publish, create_procedure_branch, \
    save_channel_changes, build_channel_list
from publisher.utils import get_procedure_code
from publisher.processing.procedure import build_single_phase, build_procedure, create_procedure_from_production,\
    update_phase_name, build_channel, update_channel, create_phase
from publisher.processing.data_sources.organisation import get_data_from_channel
from publisher.processing.data_sources.csv_steps_file import get_csv_steps_file


class Publisher(object):
    def __init__(self):
        self.procedure_code = None

    def get_channel_list(self):
        return build_channel_list()

    def get_channel_data(self, channel):
        return get_data_from_channel(channel)

    def get_procedure_list(self):
        return build_procedure_list()

    def get_phase_list(self):
        phase_list = build_phase_list()
        phase_list.sort()
        return phase_list

    def get_current_procedure(self):
        self.procedure_code = get_procedure_code()
        return self.procedure_code

    def set_current_procedure(self, proc_code):
        self.procedure_code = proc_code

    def workon_procedure(self, proc_code):
        self.check_unstaged_changes()
        change_procedure(proc_code)
        self.set_current_procedure(proc_code)

    def check_unstaged_changes(self):
        if has_unstaged_changes():
            raise UnsavedChangesError("Unsaved changes found, please contact pipeline.")

    def reset_repo(self):
        reset_repo()

    def save_working_changes(self, message):
        save_working_changes(message)

    def publish(self, message, distribution_group):
        print("-- Saving changes --")
        save_working_changes(message)
        print("-- Saved changes --")
        selected_commit = get_commits_for_procedure()[0]
        print("-- Publishing to TS-Testing --")
        publish(selected_commit.copy_with_new_note(distribution_group))
        print("-- Publish Successful --")

    def update_phase(self, phase_code, graphics, pip_graphics, widget_graphics, thumbnails, step_numbers, info_step,
                     country_restrict, csv_version):
        build_single_phase(phase_code, graphics=graphics,
                           pip_graphics=pip_graphics,
                           widget_graphics=widget_graphics,
                           thumbnails=thumbnails, step_numbers=step_numbers,
                           info_step=info_step, csv_version=csv_version)

    def update_phase_name(self, phase_code, new_name):
        update_phase_name(phase_code, new_name)

    def update_procedure(self, graphics, pip_graphics, widget_graphics, thumbnails, step_numbers, info_step,
                         country_restrict):
        build_procedure(self.procedure_code, build_phases=True,
                        graphics=graphics, pip_graphics=pip_graphics,
                        widget_graphics=widget_graphics, thumbnails=thumbnails,
                        step_numbers=step_numbers, info_step=info_step)

    def update_procedure_info(self, thumbnails):
        build_procedure(self.procedure_code, build_phases=False, graphics=False,
                        pip_graphics=False, widget_graphics=False,
                        thumbnails=thumbnails, step_numbers=False,
                        info_step=False)

    def create_procedure(self, proc_code):
        if proc_code not in build_procedure_list():
            self.check_unstaged_changes()
            create_procedure_branch(proc_code)
        else:
            raise ProcedureExistsError("The procedure {0} already exists and thus cannot be created".format(proc_code))

        create_procedure_from_production(proc_code)
        save_working_changes("Initial commit", initial=True, procedure_code=proc_code)

        self.set_current_procedure(proc_code)

    def create_phase(self, proc_code, phase_code):
        if proc_code in build_procedure_list():
            self.workon_procedure(proc_code)
        else:
            raise MissingProcedureError("The procedure {0} doesn't exist, please create it".format(proc_code))

        create_phase(proc_code, phase_code)
        save_working_changes("Added new phase {0}".format(phase_code))

        self.update_procedure_info(None)

    def create_channel(self, channel_data):
        build_channel(channel_data)
        message = "Adding channel - {}".format(channel_data['code'])
        save_channel_changes(message)

    def update_channel(self, code, updates):
        update_channel(code, updates)
        message = "Updating channel - {}".format(code)
        save_channel_changes(message)

    def get_csv_file(self, phase_code, csv_version):
        return get_csv_steps_file(phase_code, csv_version)


class UnsavedChangesError(IOError):
    pass


class ProcedureExistsError(IOError):
    pass


class MissingProcedureError(IOError):
    pass



from __future__ import absolute_import, division, print_function, unicode_literals
from past.builtins import basestring
from io import open

import os.path
import ruamel
from publisher.processing.data_sources.utils import remove_whitespace_from_string

from publisher.exceptions import ContentMissingError
from publisher.processing.data_sources.utils import YamlObject
import publisher.settings as settings


def remove_whitespace(data):
    if isinstance(data, ruamel.yaml.comments.CommentedSeq):
        for value in data:
            remove_whitespace(value)
    elif isinstance(data, ruamel.yaml.comments.CommentedMap):
        for key, value in data.items():
            if isinstance(value, basestring):
                data[key] = remove_whitespace_from_string(data[key])
            elif isinstance(value, (ruamel.yaml.comments.CommentedMap,
                                    ruamel.yaml.comments.CommentedSeq)):
                remove_whitespace(value)

    return data


def get_more_info_data(info_id, code):
    yaml_dir = os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, code, "build_assets", "moreInfo", info_id)

    yaml_file = os.path.join(yaml_dir, info_id + ".yml")
    source_asset_dir = os.path.join(yaml_dir, "assets")

    if not os.path.isfile(yaml_file):
        raise ContentMissingError("Yaml infocard not found in {0}".format(yaml_file))

    print("-- Found more info card: %s --" % info_id)
    return read_yaml_data(yaml_file), source_asset_dir


def read_yaml_data(yaml_file):
    yaml = YamlObject()

    with open(yaml_file, "r", encoding="utf-8") as f:
        data = yaml.load(f)
        return remove_whitespace(data)



from __future__ import absolute_import, division, print_function, unicode_literals

import os

from publisher.processing.data_sources.utils import YamlObject, copy_assets
import publisher.settings as settings
from .info_card import read_yaml_data


def get_overview_and_devices(code, asset_directory):
    yaml_overview_dir = os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, code, "build_assets", "overview")
    yaml_devices_dir = os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, code, "build_assets", "devices")

    yaml_overview_file = os.path.join(yaml_overview_dir, "overview.yml")
    yaml_overview_asset_dir = os.path.join(yaml_overview_dir, "assets")

    yaml_devices_file = os.path.join(yaml_devices_dir, "devices.yml")
    yaml_devices_asset_dir = os.path.join(yaml_devices_dir, "assets")

    # Overview
    if os.path.isfile(yaml_overview_file):
        print("-- Found overview yaml file: %s --" % yaml_overview_file)
        overview_data = read_yaml_data(yaml_overview_file)
    else:
        overview_data = []

    # Key Instruments
    if os.path.isfile(yaml_devices_file):
        print("-- Found devices yaml file: %s --" % yaml_devices_file)
        devices_data = read_yaml_data(yaml_devices_file)
        copy_assets(yaml_devices_asset_dir, asset_directory)
    else:
        devices_data = []

    copy_assets(yaml_overview_asset_dir, asset_directory)

    return overview_data, devices_data

